@@ -225,7 +225,7 @@ void probe_security_bprm_committed_creds(struct linux_binprm *bprm)
 #if defined(CONFIG_ARM64)
 
 /**
- * probe_security_hook_heads_on_arm - Find security_hook_heads on ARM64.
+ * probe_security_hook_heads_on_arm64 - Find security_hook_heads on ARM64.
  *
  * @base: Address of security_bprm_committed_creds().
  *
@@ -261,7 +261,7 @@ static void * __init probe_security_hook_heads_on_arm64(unsigned int *base)
 		tmp += offset;
 		/*
 		 * Find
-		 *   ldr Xt, Xn, #uimm12
+		 *   ldr Xt, [Xn, #uimm12]
 		 * sequence.
 		 */
 		for (ip += 2; i < 32 - 2; ip++, i++) {
@@ -274,6 +274,38 @@ static void * __init probe_security_hook_heads_on_arm64(unsigned int *base)
 		}
 		break;
 	}
+	for (i = 0; i < 32; ip++, i++) {
+		unsigned long tmp;
+		unsigned long offset;
+
+		/*
+		 * Find
+		 *   adrp Xd, #imm21
+		 * sequence.
+		 */
+		if ((*ip & 0x9F000000) != 0x90000000)
+			continue;
+		tmp = ((unsigned long) ip) & ~0xFFFUL;
+		offset = (unsigned long) (((((*ip >> 5) & 0x007FFFF) << 2) |
+					   ((*ip >> 29) & 0x3))) << 12;
+		if (offset & 0x100000000UL)
+			offset |= 0xFFFFFFFF00000000UL;
+		tmp += offset;
+		/*
+		 * Find
+		 *   ldr Xt, [Xn, #uimm12]
+		 * sequence.
+		 */
+		for (ip += 1; i < 32 - 1; ip++, i++) {
+			if ((*ip & 0xFFC00000) != 0xF9400000)
+				continue;
+			offset = ((*ip >> 10) & 0xFFF) << 3;
+			tmp += offset;
+			ip4ret = (unsigned int *) tmp;
+			return &ip4ret;
+		}
+		break;
+	}
 	return NULL;
 }
 
@@ -502,7 +534,47 @@ static void * __init probe_find_variable(void *function, unsigned long addr,
 	return NULL;
 }
 
-#if defined(LSM_HOOK_INIT)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(5, 8, 0)
+
+/**
+ * probe_security_hook_heads - Find address of "struct security_hook_heads security_hook_heads".
+ *
+ * Returns pointer to "struct security_hook_heads" on success, NULL otherwise.
+ */
+struct security_hook_heads * __init probe_security_hook_heads(void)
+{
+	const unsigned int offset = offsetof(struct security_hook_heads,
+					     bprm_committed_creds);
+	void *cp;
+	struct security_hook_heads *shh;
+	struct security_hook_list *entry;
+	void *cap = probe_find_symbol(" cap_bprm_creds_from_file\n");
+
+	/* Get location of cap_bprm_set_creds(). */
+	cap = check_function_address(cap, "cap_bprm_creds_from_file");
+	if (!cap)
+		return NULL;
+	/* Guess "struct security_hook_heads security_hook_heads;". */
+	cp = probe_find_variable(probe_security_bprm_committed_creds,
+				 ((unsigned long)
+				  &probe_dummy_security_hook_heads) + offset,
+				 " security_bprm_committed_creds\n");
+	if (!cp) {
+		printk(KERN_ERR
+		       "Can't resolve security_bprm_committed_creds().\n");
+		return NULL;
+	}
+	/* This should be "struct security_hook_heads security_hook_heads;". */
+	shh = ((void *) (*(unsigned long *) cp)) - offset;
+	hlist_for_each_entry(entry, &shh->bprm_creds_from_file, list)
+		if (entry->hook.bprm_creds_from_file == cap)
+			return shh;
+	printk(KERN_ERR "Guessed security_hook_heads is 0x%lx\n",
+	       (unsigned long) shh);
+	return NULL;
+}
+
+#elif defined(LSM_HOOK_INIT)
 
 /**
  * probe_security_hook_heads - Find address of "struct security_hook_heads security_hook_heads".
